﻿#region License
/* Copyright 2004, 2013 James F. Bellinger
 * 
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redestribute it
 * freely, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must
 * not claim that you wrote the original software. If you use this software
 * in a product, an acknowledgement in the product documentation would be
 * appreciated but is not required.
 * 
 * 2. Altered source versions must be plainly marked as such, and must
 * not be misrepresented as being the original software.
 * 
 * 3. This notice may not be removed or altered from any source
 * distribution. */
#endregion

using System;

namespace Chatter.Plugins.Events
{
    /// <summary>
    /// Stores information about a received message.
    /// </summary>
    public class MessageEventArgs : EventArgs
    {
        /// <summary>
        /// Constructs a chat message event.
        /// </summary>
        /// <param name="sender">The sender of the message.</param>
        /// <param name="recipient">The recipient of the message.</param>
        /// <param name="message">The message text.</param>
        /// <param name="messageType">The type of message received.</param>
        /// <param name="messageFlags">Modifiers affecting the message.</param>
        public MessageEventArgs(string sender, string recipient,
            string message, MessageType messageType, MessageFlags messageFlags)
        {
            Sender = sender; Recipient = recipient; Message = message;
            MessageType = messageType; MessageFlags = messageFlags;

            IsChannelMessage = recipient.StartsWith("#");
        }

        public bool IsChannelMessage
        {
            get;
            private set;
        }

        public bool IsLocal
        {
            get;
            private set;
        }

        public string Sender
        {
            get;
            private set;
        }

        public string Recipient
        {
            get;
            private set;
        }

        public string Message
        {
            get;
            private set;
        }

        public MessageFlags MessageFlags
        {
            get;
            private set;
        }

        public MessageType MessageType
        {
            get;
            private set;
        }
    }
}