﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Chatter.Plugins
{
    /// <summary>
    /// Provides functions useful for string manipulation.
    /// </summary>
    public static class StringUtility
    {
        /// <summary>
        /// Computes the distance between two strings.
        /// </summary>
        /// <param name="firstString">The string to compare.</param>
        /// <param name="secondString">The string to compare against.</param>
        /// <returns>Returns a number between zero and one hundred, zero indicating
        /// no similarity between the strings, and one hundred indicating identical
        /// strings.</returns>
        /// <remarks>
        ///     This function was ported to C# from John Ratcliff's C++
        ///     Gestalt fuzzy string comparison class. You may visit his website at
        ///     http://codesuppository.blogspot.com/.
        /// </remarks>
        public static int StringDistance(string firstString, string secondString)
        {
            int Length1 = firstString.Length;
            int Length2 = secondString.Length;

            if (firstString == secondString) { return 100; }

            return 200 * GestaltSubstr(firstString, 0, Length1,
                secondString, 0, Length2) / (Length1 + Length2);
        }

        static int GestaltSubstr(string String1, int Start1, int End1,
                string String2, int Start2, int End2)
        {
            if (End1 <= Start1 || End2 <= Start2) { return 0; }
            if (End1 == Start1 + 1 && End2 == Start2 + 1) { return 0; }

            int IStart1 = 0, IStart2 = 0, IEnd1 = End1, IEnd2 = End2, Max = 0;

            for (int Pos1 = Start1; Pos1 < End1; Pos1++)
            {
                for (int Pos2 = Start2; Pos2 < End2; Pos2++)
                {
                    if (String1[Pos1] == String2[Pos2])
                    {
                        for (int Loc = 1;
                            Pos1 + Loc < String1.Length &&
                            Pos2 + Loc < String2.Length &&
                            String1[Pos1 + Loc] == String2[Pos2 + Loc];
                            Loc++)
                        {
                            if (Loc > Max)
                            {
                                Max = Loc;
                                IStart1 = Pos1; IStart2 = Pos2;
                                IEnd1 = End1 - Max; IEnd2 = End2 - Max;
                            }
                        }
                    }
                }
            }

            if (Max == 0) { return 0; }

            Max += GestaltSubstr(String1, IStart1 + Max, End1,
                String2, IStart2 + Max, End2);
            Max += GestaltSubstr(String1, Start1, IStart1,
                String2, Start2, IStart2);

            return Max;
        }
    }
}
