#region License
/* Copyright 2004 James F. Bellinger
 * 
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redestribute it
 * freely, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must
 * not claim that you wrote the original software. If you use this software
 * in a product, an acknowledgement in the product documentation would be
 * appreciated but is not required.
 * 
 * 2. Altered source versions must be plainly marked as such, and must
 * not be misrepresented as being the original software.
 * 
 * 3. This notice may not be removed or altered from any source
 * distribution. */
#endregion

using System;
using System.Drawing;
using System.Windows.Forms;
using Chatter.Plugins;

namespace Chatter
{
	public class Channel : System.Windows.Forms.Form
	{
		private System.Windows.Forms.RichTextBox chatText;
		private System.ComponentModel.Container components = null;
		
		private string channel;
		private System.Windows.Forms.ListBox userList;
		private System.Windows.Forms.TextBox textEntry;
		private System.Windows.Forms.ContextMenu mnuUser;
		private System.Windows.Forms.MenuItem mnuUserPrivateChat;
		private System.Windows.Forms.MenuItem mnuUserPing;
		private System.Windows.Forms.ContextMenu mnuChat;
		private System.Windows.Forms.MenuItem mnuChatClear;
		private ChatterMDI main;

		public Channel(ChatterMDI main, string channel)
		{
			// NOTE: Scrolling works because HideSelection is set to 'false'.
			InitializeComponent();

			this.main = main; this.channel = channel;
			this.Closed += new EventHandler(Channel_Closed);
			this.textEntry.KeyPress += new KeyPressEventHandler(textEntry_KeyPress);
			this.chatText.SelectionHangingIndent = 8;

			this.chatText.Font  = main.Fonts.ChatText;
			this.textEntry.Font = main.Fonts.ChatText;
			this.userList.Font  = main.Fonts.UserList;

			UpdateTitle();
		}

		string FilterPrePostNames(string Name)
		{
			return Name[0] == '+' || Name[0] == '@' ?
				Name.Substring(1) : Name;
		}

		string FilterOnlyPreName(string Name)
		{
			return Name[0] == '+' || Name[0] == '@' ?
				Name[0].ToString() : "";
		}

		bool AllowResponse()
		{
            return !IsStatusWindow();
		}

        public bool IsStatusWindow()
        {
            return channel.StartsWith("**:");
        }

		public bool IsChannelWindow()
		{
			return channel.StartsWith("#");
		}

		public void UpdateTitle()
		{
			if (!AllowResponse())
			{
				Text = channel.Substring(3); userList.Visible = false;
			}
			else if (IsChannelWindow())
			{
				Text = channel; userList.Visible = true;
			}
			else
			{
				Text = channel; userList.Visible = false;
			}
		}

		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				if(components != null)
				{
					components.Dispose();
				}
			}

			base.Dispose(disposing);
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.chatText = new System.Windows.Forms.RichTextBox();
            this.mnuChat = new System.Windows.Forms.ContextMenu();
            this.mnuChatClear = new System.Windows.Forms.MenuItem();
            this.userList = new System.Windows.Forms.ListBox();
            this.mnuUser = new System.Windows.Forms.ContextMenu();
            this.mnuUserPing = new System.Windows.Forms.MenuItem();
            this.mnuUserPrivateChat = new System.Windows.Forms.MenuItem();
            this.textEntry = new System.Windows.Forms.TextBox();
            this.SuspendLayout();
            // 
            // chatText
            // 
            this.chatText.ContextMenu = this.mnuChat;
            this.chatText.Dock = System.Windows.Forms.DockStyle.Fill;
            this.chatText.HideSelection = false;
            this.chatText.Location = new System.Drawing.Point(0, 0);
            this.chatText.Name = "chatText";
            this.chatText.ReadOnly = true;
            this.chatText.ScrollBars = System.Windows.Forms.RichTextBoxScrollBars.Vertical;
            this.chatText.Size = new System.Drawing.Size(336, 226);
            this.chatText.TabIndex = 1;
            this.chatText.Text = "";
            this.chatText.LinkClicked += new System.Windows.Forms.LinkClickedEventHandler(this.chatText_LinkClicked);
            // 
            // mnuChat
            // 
            this.mnuChat.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.mnuChatClear});
            // 
            // mnuChatClear
            // 
            this.mnuChatClear.Index = 0;
            this.mnuChatClear.Text = "&Clear";
            this.mnuChatClear.Click += new System.EventHandler(this.mnuChatClear_Click);
            // 
            // userList
            // 
            this.userList.ContextMenu = this.mnuUser;
            this.userList.Dock = System.Windows.Forms.DockStyle.Right;
            this.userList.Location = new System.Drawing.Point(336, 0);
            this.userList.Name = "userList";
            this.userList.Size = new System.Drawing.Size(104, 226);
            this.userList.Sorted = true;
            this.userList.TabIndex = 2;
            this.userList.MouseDown += new System.Windows.Forms.MouseEventHandler(this.userList_MouseDown);
            // 
            // mnuUser
            // 
            this.mnuUser.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.mnuUserPing,
            this.mnuUserPrivateChat});
            // 
            // mnuUserPing
            // 
            this.mnuUserPing.Index = 0;
            this.mnuUserPing.Text = "&Ping";
            this.mnuUserPing.Click += new System.EventHandler(this.mnuUserPing_Click);
            // 
            // mnuUserPrivateChat
            // 
            this.mnuUserPrivateChat.Index = 1;
            this.mnuUserPrivateChat.Text = "Private &chat";
            this.mnuUserPrivateChat.Click += new System.EventHandler(this.mnuUserPrivateChat_Click);
            // 
            // textEntry
            // 
            this.textEntry.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.textEntry.Location = new System.Drawing.Point(0, 226);
            this.textEntry.Name = "textEntry";
            this.textEntry.Size = new System.Drawing.Size(440, 20);
            this.textEntry.TabIndex = 0;
            // 
            // Channel
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(440, 246);
            this.Controls.Add(this.chatText);
            this.Controls.Add(this.userList);
            this.Controls.Add(this.textEntry);
            this.Name = "Channel";
            this.Text = "Channel";
            this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
            this.ResumeLayout(false);
            this.PerformLayout();

		}
		#endregion

		private void Channel_Closed(object o, EventArgs e)
		{
			main.LeaveChannel(channel);
		}

		private string ShowAndFilterMessage(string Header, string Target,
			string Message)
		{
			Message = main.ComputeFilteredMessage(Message);

			if (Target != null)
			{
				ChatEntry(Color.Navy, (Header + " [" + Target.Trim() + "] -> " +
					Message).Trim());
			}
			else
			{
				ChatEntry(Color.Navy, (Header + " -> " + Message).Trim());
			}

			return Message;
		}

		private void ProcessMessage(string Message)
		{
			if (Message.StartsWith("/"))
			{
				string[] Split = Message.Trim().Split(new char[] { ' ' }, 2);

				switch(Split[0].ToLower())
				{
					case "/j": case "/join":
						if (Split.Length < 2) { goto default; }
						main.JoinChannel(Split[1].Trim());
						break;

					case "/a": case "/action":
						if (Split.Length < 2) { goto default; }
						main.Connection.ChatAction(channel, ShowAndFilterMessage
							("Action", null, Split[1]));
						break;

					case "/list":
						main.Connection.ListChannels(); break;

					case "/m": case "/msg":
					case "/n": case "/notice":
					{
						if (Split.Length < 2) { goto default; }

						string[] Subsplit = Split[1].Split(new char[] { ' ' }, 2);
						if (Subsplit.Length < 2) { goto default; }

						if (char.ToLower(Split[0][1]) == 'n')
						{
							main.Connection.Notice(Subsplit[0].Trim(),
								ShowAndFilterMessage("Notice", Subsplit[0],
								Subsplit[1]));

							main.InvokePluginMessage(main.Nickname, Subsplit[0].Trim(),
								                     Subsplit[1].Trim(), MessageType.Notice, MessageFlags.Local);
						}
						else
						{
							main.Connection.Message(Subsplit[0].Trim(),
								ShowAndFilterMessage("Message", Subsplit[0],
								Subsplit[1]));

                            main.InvokePluginMessage(main.Nickname, Subsplit[0].Trim(),
                                                     Subsplit[1].Trim(), MessageType.Message, MessageFlags.Local);
						}
					}
						break;

					default: ChatEntry(Color.DarkKhaki, "Unrecognized command syntax"); break;
				}
			}
			else if (AllowResponse())
			{
				main.Connection.Message(channel, ShowAndFilterMessage
					("", null, Message));

                main.InvokePluginMessage(main.Nickname, channel,
                                         Message, MessageType.Message, MessageFlags.Local);
			}
		}

		private void textEntry_KeyPress(object o, KeyPressEventArgs e)
		{
			if (e.KeyChar == '\r')
			{
				if (textEntry.Text.Length > 0)
				{
					string Message = textEntry.Text.Trim();

					if (Message.Length > 0)
					{
						ProcessMessage(Message);
					}

					textEntry.Clear(); e.Handled = true;
				}
			}
		}

		public void UserAdd(string[] Name)
		{
			userList.Items.AddRange(Name);
		}

		public void UserRemove(string Name)
		{
			userList.Items.Remove(Name);
		}

		public void ChatEntry(Color messageColor, string messageText)
		{
			chatText.SelectionStart = chatText.TextLength;
			chatText.SelectionColor = messageColor;
			chatText.SelectedText =
				messageText.Replace("\r", "").Replace("\n", "").Replace("\0", "") +
				"\r\n";
			chatText.ScrollToCaret();
		}

		private void mnuUserPrivateChat_Click(object sender, System.EventArgs e)
		{
			string s = (string)userList.SelectedItem;

			if (s != null) { main.FormChatWindow(FilterPrePostNames(s)); }
		}

		private void mnuUserPing_Click(object sender, System.EventArgs e)
		{
			string s = (string)userList.SelectedItem;

			if (s != null)
			{
				main.Connection.Ping(FilterPrePostNames(s));
			}
		}

		public bool NameChangeRawBoth(string OldName, string NewName)
		{
			for (int i = 0; i < userList.Items.Count; i ++)
			{
				if ((string)userList.Items[i] == OldName)
					{ userList.Items[i] = NewName; return true; }
			}

			return false;
		}

		public bool NameChangeRawNew(string OldName, string NewName)
		{
			for (int i = 0; i < userList.Items.Count; i ++)
			{
				if (FilterPrePostNames((string)userList.Items[i]) == OldName)
					{ userList.Items[i] = NewName; return true; }
			}

			return false;
		}

		public bool NameChange(string OldName, string NewName)
		{
			for (int i = 0; i < userList.Items.Count; i ++)
			{
				if (FilterPrePostNames((string)userList.Items[i]) == OldName)
				{
					userList.Items[i] = FilterOnlyPreName
						((string)userList.Items[i]) + NewName;
					return true;
				}
			}

			return false;
		}

		private void mnuChatClear_Click(object sender, System.EventArgs e)
		{
			chatText.Clear();
		}

		private void chatText_LinkClicked(object sender,
			System.Windows.Forms.LinkClickedEventArgs e)
		{
            try
            {
                System.Diagnostics.Process.Start(e.LinkText);
            }
            catch (Exception)
            {
                MessageBox.Show(this,
                                "Something went wrong while trying to follow that link.",
                                "Oops", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
		}

        private void userList_MouseDown(object sender, MouseEventArgs e)
        {
            int index = userList.IndexFromPoint(e.X, e.Y);
            if (index != -1) { userList.SelectedIndex = index; }
        }
	}
}
