#region License
/* Copyright 2004 James F. Bellinger
 * 
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redestribute it
 * freely, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must
 * not claim that you wrote the original software. If you use this software
 * in a product, an acknowledgement in the product documentation would be
 * appreciated but is not required.
 * 
 * 2. Altered source versions must be plainly marked as such, and must
 * not be misrepresented as being the original software.
 * 
 * 3. This notice may not be removed or altered from any source
 * distribution. */
#endregion

using System;
using System.IO;
using System.Xml.Serialization;

namespace Chatter.Logic
{
	public struct IdentdSettings
	{
		public bool   Enabled;
		public string OperatingSystem;
		public string UserID;
	}

	public struct UserSettings
	{
		public string Nickname;
		public string RealName;
	}

	public class ServerSettings
	{
        public string Default = "Freenode";

        public ServerEntry[] Listing = new[]
        {
            new ServerEntry() { Host = "irc.freenode.net", Description = "Freenode", PortRange = "7070", SSL = true },
            new ServerEntry() { Host = "irc.oftc.net", Description = "OFTC" },
            new ServerEntry() { Host = "irc.rizon.net", Description = "Rizon" },
            new ServerEntry() { Host = "irc.quakenet.org", Description = "QuakeNet", PortRange = "6667-6669" }
        };
	}

	public struct FontSetting
	{
		public string Name;
		public float  Size;
	}

	public struct FontSettings
	{
		public FontSetting Channels;
		public FontSetting ChatText;
		public FontSetting UserList;
	}

	public class ServerEntry: IComparable
	{
		public string Host;
		public string Description;
        public string PortRange = "6667";
        public bool SSL;

		public override string ToString()
		{
			if (Description == "") { return Host; }
			else if (Host == "") { return Description; }
			else { return Description + " - " + Host; }
		}

		public int CompareTo(object o)
		{
			return ToString().CompareTo(o.ToString());
		}
	}

	public struct FilterOptions
	{
		public bool Elizabethan;
		public bool ProperEnglish;
		public bool Shorthand;
	}

	public class LanguageOptions
	{
		public FilterOptions Filters;
	}

	public sealed class Settings
	{
        public LanguageOptions Language = new LanguageOptions();

        public IdentdSettings Identd = new IdentdSettings()
        {
            Enabled = true,
            OperatingSystem = "UNIX",
            UserID = "johndoe"
        };

        public UserSettings UserInfo = new UserSettings()
        {
            Nickname = "YourNameHere",
            RealName = "John Doe"
        };

        public ServerSettings Servers = new ServerSettings();

        public FontSettings Fonts = new FontSettings()
        {
            Channels = new FontSetting() { Name = "Verdana", Size = 9.75f },
            ChatText = new FontSetting() { Name = "Verdana", Size = 9.75f },
            UserList = new FontSetting() { Name = "Verdana", Size = 9.75f }
        };

        static Settings()
        {
            object settings;
            SettingsIO.TryLoad(Directories.ProgramSettingsFilename, typeof(Settings), out settings);
            Instance = (Settings)settings ?? new Settings();
        }

        Settings()
        {

        }

		public void Save()
		{
            SettingsIO.TrySave(Directories.ProgramSettingsFilename, this);
		}

        public static Settings Instance
        {
            get;
            private set;
        }
	}
}