﻿#region License
/* Copyright 2013 James F. Bellinger
 * 
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redestribute it
 * freely, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must
 * not claim that you wrote the original software. If you use this software
 * in a product, an acknowledgement in the product documentation would be
 * appreciated but is not required.
 * 
 * 2. Altered source versions must be plainly marked as such, and must
 * not be misrepresented as being the original software.
 * 
 * 3. This notice may not be removed or altered from any source
 * distribution. */
#endregion

using System;
using System.IO;
using System.Xml.Serialization;

namespace Chatter.Logic
{
    static class SettingsIO
    {
        public static bool TryLoad(string path, Type type, out object settings)
        {
            settings = null;

            try { Directory.CreateDirectory(Path.GetDirectoryName(path)); }
            catch (Exception) { return false; }

            Stream stream;
            try { stream = File.Open(path, FileMode.Open, FileAccess.Read); }
            catch (Exception) { return false; }

            using (stream)
            {
                try
                {
                    var serial = new XmlSerializer(type);
                    settings = serial.Deserialize(stream);
                }
                catch (InvalidOperationException)
                {
                    return false;
                }

                return true;
            }
        }

        public static bool TrySave(string path, object settings)
        {
            Stream stream;
            try { stream = File.Create(path); }
            catch (Exception) { return false; }

            using (stream)
            {
                try
                {
                    var serial = new XmlSerializer(settings.GetType());
                    serial.Serialize(stream, settings);
                }
                catch (Exception)
                {
                    return false;
                }

                return true;
            }
        }
    }
}
